% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function [X,Y] = myAlterProj(A,W,tol)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = size(A);
n = n(1);

Sp = zeros(n,n);
Yp = A;
Winv = inv(W);

itr = 0;
itr_tol = [];
itr_abs_err=[];
dA = eig(A);
itr_eig(1) = min(dA);
while(1)
  R = Yp - Sp;
  X = projAtoS(R);
  S = X - R;
  Y = projAtoU(X, Winv);
  t = norm(Y-Yp,inf);
  t2 = norm(Yp,inf);
  if(t2<=0)
    disp('negative norm!');
    return;
  end
  cur_tol = t/t2;
  if(cur_tol < tol)
    msg = sprintf('total itr = %d', itr);
    disp(msg);
    % relative error plot w.r.t. iterations
    figure;
    semilogy(itr_tol, 'r.'); 
    % absolute error plot w.r.t. iterations
    figure;
    semilogy(itr_abs_err,'r.');
    % eigen-value plot w.r.t. iterations
    figure;
    semilogy(-itr_eig, 'r.');
    xlabel('Iteration number k');
    ylabel('log(-\lambda(Y_k)_{least})');
    % rate of convergence plot w.r.t. iterations
    rate = itr_eig(2:itr)./itr_eig(1:itr-1);
    figure;
    plot(rate,'r.');
    xlabel('Iteration number k');
    ylabel('\lambda(Y_{k+1})_{least}/\lambda(Y_{k})_{least}');
    
    return;
  end
  itr = itr+1;
  itr_tol(itr) = cur_tol;
  itr_abs_err(itr) = t;
  dY = eig(Y);
  itr_eig(itr+1) = min(dY);
  Yp = Y;
end 


function [Aplus] = projAtoS(A)
[Q,D] = eig(A);
theta = diag(D);
postheta = max(theta,0);
Aplus = Q*diag(postheta)*Q';


function [Au] = projAtoU(A,Winv)
Yeye = eye(size(A));
b = diag(A-Yeye);
theta = (Winv.*Winv)\b;
Au = A - Winv*diag(theta)*Winv;
