% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function myAlterTest(numPt, pwlPar)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

initRandState;
tol = 10^-8;

chipX=10;
chipY=10;
stepN = 5;

%A = myFindNonPhyCorr(n);
step_flag = 1;
if step_flag == 1
A = myFindNonPhyCorrFromStep(numPt, chipX, chipY, stepN);
end
pwl_flag = 0;
if pwl_flag == 1
A = myFindNonPhyCorrFromPWL2(numPt, chipX, chipY, pwlPar);
end

dA = eig(A);
mindA = min(dA)

W = eye(size(A));
[X,Y] = myAlterProj(A,W,tol);
dY = eig(Y);
mindY = min(dY)

diffMat = A - Y;
divMat = A;
[fomDiff,relDiff] = myNormDiff(diffMat, divMat)



function covMStep = myFindNonPhyCorrFromStep(numPt, chipX, chipY, stepN)
xmax = chipX*sqrt(2);
par = [xmax, stepN]'; 
itr = 0;
p = 0;
while (p==0)
    [x1V, y1V] = mychipRndSample(numPt, chipX, chipY);
    [v, rhovStep, covMStep] = mySpatialEvalCorrelation(x1V, y1V, @mySpatialStepFunc, par);
    [R,p] = chol(covMStep);
    itr = itr+1;
    msg = sprintf('Iteration %d', itr);
    disp(msg);
end

function covMStep = myFindNonPhyCorrFromPWL2(numPt, chipX, chipY, par)
itr = 0;
p = 0;
while (p==0)
    [x1V, y1V] = mychipRndSample(numPt, chipX, chipY);
    v = myPt2DistVect(x1V, y1V);
    rhov = myPWL2FittingFunc(par,v);
    covMPWL = myVect2CovM(rhov, x1V, y1V);
    [R,p] = chol(covMPWL);
    
    itr = itr+1;
    msg = sprintf('Iteration %d', itr);
    disp(msg);
end

function A = myFindNonPhyCorr(n)
itr = 0;
p = 0;
while(p==0)
    A = rand(n);
    for i=1:n
        A(i,i) = 1;
    end
    [R,p] = chol(A);
    itr = itr+1;
    msg = sprintf('Iteration %d', itr);
    disp(msg);
end

function [fomDiff,relDiff] = myNormDiff(diffMat, divMat)
fomDiff= [norm(diffMat,1), norm(diffMat,2),...
              norm(diffMat,inf), norm(diffMat,'fro'), max(max(diffMat))];
relDiff= [fomDiff(1)/norm(divMat,1),fomDiff(2)/norm(divMat,2), ...
              fomDiff(3)/norm(divMat,inf), fomDiff(4)/norm(divMat,'fro'), max(max(diffMat))];
          
