% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function rhov = myBesselFittingFunc(par,v)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Given scaler b and s, output the correlation function
% rho(v) for the given v, which can either be a scaler or a vector
% Example usage:
%>> b = -0.1;
%>> s =2;
%>> par = [b,s]';
%>> vVect = [0:0.1:10];
%>> myBesselPlot(par,vVect)
b = par(1);
s = par(2);
n = length(v);

bv = b*v;
ksm = besselk(s-1,bv);
%if(sum(isfinite(ksm))~=n)
%  idxM = myFindFirstNonValidNum(ksm);
%end
gsm = gamma(s-1);
%if(sum(isfinite(gsm))~=n)
%  idxM = myFindFirstNonValidNum(gsm);
%end

rhov = 2*(bv/2).^(s-1);
%if(sum(isfinite(rhov))~=n)
%  idxM = myFindFirstNonValidNum(rhov);
%end
rhov = rhov.*ksm;
%if(sum(isfinite(rhov))~=n)
%  idxM = myFindFirstNonValidNum(rhov);
%end
rhov = rhov/gsm;
%if(sum(isfinite(rhov))~=n)
%  idxM = myFindFirstNonValidNum(rhov);
%end
%cntInf = sum(isinf(rhov));
%cntNan = sum(isnan(rhov));
%str = sprintf('b=%f, s=%f, Inf Cnt = %d, Nan Cnt = %d', b, s, cntInf, cntNan);
%disp(str);

function idxM = myFindFirstNonValidNum(val)
n = length(val);
flag = isfinite(val);
idxM = [];
idx = 1;
for i=1:n
  if(flag(i)==0)
    idxM(idx) = i;
    idx = idx+1;
  end
end

