% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function rhov = myPWL2FittingFunc(par,v)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Given piece-wise linear function defined by par, where
% x(2:segN) = par(1:segN-1),  x(0) = 0, x(segN+1) = max(v)
% y(1:segN+1)=par(segN:N), and (x,y) are the turning points for PWL,
% evaluate function value at v.
% Example usage:

N = length(par);
segN = N/2;

X = zeros(segN+1);
Y = zeros(segN+1);
X(1) = 0;  % alwasy fix the first point to be 0
X(segN+1) = max(v);
X(2:segN) = par(1:segN-1);
Y(1:segN+1) = par(segN:N);

n = length(v);
rhov = zeros(n,1);
for i=1:n
    x = v(i);
    if(x==X(1))
        rhov(i) = Y(1);
    elseif(x==X(segN+1))
        rhov(i) = Y(segN+1);
    else
        idx = myFindStepLoc(X,x);
        x1 = X(idx);
        x2 = X(idx+1);
        y1 = Y(idx);
        y2 = Y(idx+1);
        y = y2 - (x2 - x)/(x2-x1) * (y2-y1);
        rhov(i) = y;
    end
end


function idx = myFindStepLoc(X, x)    
ptN = length(X);
for i=2:ptN
    if(x<X(i))
        idx = i-1;
        return;
    end
end
disp('error');
