% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function [parOut, rhovOut, v, rhov] = myPWLFuncFitting(M, pwlN)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Given the data M=[v, rhov], and the given PWL segment number, pwlN,
% we try to find the best fit of the data. Assume PWL is monotically
% decreasing
%
% M = csvread(fn);
v = M(:,1);
rhov=M(:,2);
xmax = max(v);
xmin = min(v);
ymax = max(rhov);
ymin = min(rhov);
xavg = (xmax - xmin)/pwlN;
yavg = (ymax - ymin)/pwlN;

par = zeros(pwlN+1,2);
par(1,1) = xmin; 
par(1,2) = ymax;
for i=2:pwlN+1
    x = xmin + (i-1)*xavg;
    y = ymax - (i-1)*yavg;
    par(i,1) = x;
    par(i,2) = y;
end

[parOut,resnorm,residual] = lsqcurvefit(@myPWLFittingFunc, par, v, rhov);
verV = linspace(0,xmax,1000);
rhovOut = myPWLFittingFunc(parOut, verV);
figure;
plot(v,rhov,'r.');
hold on;
plot(v,rhovOut,'b', 'LineWidth', 2);



