% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function myRunScript
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

numPt=200;
chipX=10;
chipY=10;
xmax = chipX*sqrt(2);
[x1V, y1V] = mychipRndSample(numPt, chipX, chipY);
figure; plot(x1V, y1V,'r.');
% generate the data according to the linear decreasing spatial correlation
% function
par=[xmax];
[v, rhovLin, covMLin] = mySpatialEvalCorrelation(x1V, y1V, @mySpatialLinFunc, par);
stepN = 5;
par = [xmax, stepN]'; 
[v, rhovStep, covMStep] = mySpatialEvalCorrelation(x1V, y1V, @mySpatialStepFunc, par);

% Curve fitting to get the best fit of spatial correlation function
[parOutLin, rhovOutLin] = mySpatialFuncFitting([v,rhovLin]);
covMBS_Lin = myVect2CovM(rhovOutLin, x1V, y1V);

[parOutStep, rhovOutStep] = mySpatialFuncFitting([v,rhovStep]);
covMBS_Step = myVect2CovM(rhovOutStep, x1V, y1V);

% Use the alternative projection method to find the least square
% error correlation matrix
[covX_Lin,covY_Lin] = myAlterProj(covMLin,eye(size(covMLin)),10^-8);
[covX_Step,covY_Step] = myAlterProj(covMStep,eye(size(covMStep)),10^-8);

figure; hold;
plot(v,rhovLin,'r.');
plot(v,rhovOutLin, 'b+');
plot(v,rhovStep, 'ko');
plot(v,rhovOutStep, 'g*');
legend('Lin', 'Fit4Lin', 'Step', 'Fit4Step');

figure; hold;
plot(v,rhovLin,'r.');
plot(v,rhovOutLin, 'b+');
legend('Lin', 'Fit4Lin');

figure; hold;
plot(v,rhovStep, 'k.');
plot(v,rhovOutStep, 'g*');
legend('Step', 'Fit4Step');

diffR2F_Lin = covMLin - covMBS_Lin;
diffR2F_Step = covMStep - covMBS_Step;
diffR2Ax_Lin = covMLin - covX_Lin;
diffR2Ay_Lin = covMLin - covY_Lin;
diffR2Ax_Step = covMStep - covX_Step;
diffR2Ay_Step = covMStep - covY_Step;
[fomR2F_Lin,fomR2F_Lin_rel] = myNormDiff(diffR2F_Lin, covMLin)
[fomR2F_Step,fomR2F_Step_rel] = myNormDiff(diffR2F_Step, covMStep)
[fomR2Ax_Lin,fomR2Ax_Lin_rel] = myNormDiff(diffR2Ax_Lin, covMLin)
[fomR2Ay_Lin,fomR2Ay_Lin_rel] = myNormDiff(diffR2Ay_Lin, covMLin)
[fomR2Ax_Step,fomR2Ax_Step_rel] = myNormDiff(diffR2Ax_Step, covMStep)
[fomR2Ay_Step,fomR2Ay_Step_rel] = myNormDiff(diffR2Ay_Step, covMStep)


function [fomDiff,relDiff] = myNormDiff(diffMat, divMat)
fomDiff= [norm(diffMat),norm(diffMat,1), norm(diffMat,2),...
              norm(diffMat,inf), norm(diffMat,'fro'), max(max(diffMat))];
relDiff= [fomDiff(1)/norm(divMat),fomDiff(2)/norm(divMat,1), fomDiff(3)/norm(divMat,2),...
              fomDiff(4)/norm(divMat,inf), fomDiff(5)/norm(divMat,'fro'), max(max(diffMat))];
          

