% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function [per_g_err, per_s_err, per_r_err, per_rho_err]=myVarDecomp(inputSampleN, inputNoiseRatio, inputSiteN)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

initRandState;

if nargin==2
    inputSiteN = 1000;
end

%%sitePt = 1000;  % number of sites used to extract variation part
sitePt = inputSiteN;
% numPt=30;  % number of points used to extract spatial correlation function
numPt=sitePt;  % number of points used to extract spatial correlation function
chipX=10;
chipY=10;

b=2;  %underlying spatial correlation function
s=10;

var_g = 9;  sigma_g = sqrt(var_g);% global variation
var_s = 25;  sigma_s = sqrt(var_s);% spatial variation
var_r = 9;  sigma_r = sqrt(var_r); % random variation
var_total = var_g+var_s+var_r;
msg = sprintf('Underlying: var(glb)=%f, var(spa)=%f, var(rnd)=%f',var_g,var_s,var_r);
disp(msg);
msg = sprintf('Underlying: var(total)=%f, var(glb)=%f, var(spatial+rnd)=%f',...
    var_total, var_g, var_s+var_r);
disp(msg);

noiseFlag_gen = 0;  % whether or not to add noise to the sampling to the first part
glbNoiseFlag_gen = 0;
noiseSigma = sqrt(inputNoiseRatio*var_total);  % noise is about 50% of total variation
glbNoiseSigma = 0.25*sigma_r; % glb noise

%sampleN = 1000; % # of chips
sampleN = inputSampleN; % # of chips

% generate global variation samples
gndSample = normrnd(0,sigma_g,[sampleN, 1]);

var_flag = 1;
if(var_flag == 1)
  %% generate data for variation extraction
  %% assuming all measurement sites within a chip are far away
  %% so that there is no spatial correlation between them  
  f_i_k = zeros(sampleN, sitePt);
  for i=1:sampleN    
      gndS = gndSample(i);
      if(glbNoiseFlag_gen==1)
          gndS = gndS + normrnd(0,glbNoiseSigma);
      end
      for k=1:sitePt
          rndS = normrnd(0,sigma_r);
          spaS = normrnd(0,sigma_s);
          f_i_k(i,k) = spaS + rndS + gndS;
          if(noiseFlag_gen==1)
              nosS = normrnd(0,noiseSigma);
              f_i_k(i,k) = f_i_k(i,k) + nosS;
          end        
      end
  end
  f_i_k_sqr = f_i_k.^2;
  f_sum = sum(sum(f_i_k));
  f_sqr_sum = sum(sum(f_i_k_sqr));
  est_sig_F_sqr = (f_sqr_sum - f_sum*f_sum/sampleN/sitePt)/(sampleN*sitePt-1);

  f_chip_dot_sum = sum(f_i_k, 2);
  f_sqr_chip_dot = sum(f_i_k_sqr, 2);
  est_sig_Fchip_sqr = (f_sqr_chip_dot - f_chip_dot_sum.^2/sitePt)/(sitePt-1);
  est_sig_Fchip_sqr = sum(est_sig_Fchip_sqr)/sampleN;
  est_sig_g_sqr = est_sig_F_sqr - est_sig_Fchip_sqr;
else
  est_sig_F_sqr = 42.9112;
  est_sig_g_sqr = 8.8982;
  est_sig_Fchip_sqr = 34.0131;
end
msg = sprintf('Estimated: var(total)=%f, var(glb)=%f, var(spatial+rnd)=%f',...
    est_sig_F_sqr, est_sig_g_sqr, est_sig_Fchip_sqr);
disp(msg);

par_in = [b, s]';

datFromChipPtFlag = 1;
if(datFromChipPtFlag==1)
  % generate sample points from chips, which in turn determines the
  % distance
  [x1V, y1V] = mychipRndSample(numPt, chipX, chipY);
  %plot(x1V,y1V,'r.');
  % compute the correlation coefficient for the given distance v
  [v, rhov] = mySpatialEvalCorrelation(x1V, y1V, @myBesselFittingFunc, par_in);
  % put correlation into a covariance matrix format
  spcovM = myVect2CovM(rhov, x1V, y1V);
else
   % generate sample points with specified distance
   v_max = sqrt(2)*chipX;
   v_min = 0.01;
   x_ptN = 50;   
   v_ptN = x_ptN*(x_ptN-1)/2;
   v = linspace(v_min, v_max, v_ptN);
   rhov = myBesselFittingFunc(par_in,v);
   % put correlation into a covariance matrix format
   spcovM = myVect2CovM(rhov, x_ptN);
end

model_cov_f_i_j = rhov*var_s + var_g;  % model covariance cov(Fi,Fj)
model_cor_i_j = model_cov_f_i_j/(var_g+var_s+var_r); %model correlation coeff

% generate the measurement data = spatial + global + rand + noise(?)
% where spatial variation follow certain correlation 
noiseFlag_spa = 1;
glbNoiseFlag_spa = 0;
spcovMvar = spcovM.*var_s;  % get the spatial variance matrix
spmuV = zeros(1,numPt);
spSample = mvnrnd(spmuV,spcovMvar,sampleN);
for i=1:sampleN    
    gndS = gndSample(i);
    if(glbNoiseFlag_spa==1)
        gndS = gndS + normrnd(0,glbNoiseSigma);
    end
    for k=1:numPt
        rndS = normrnd(0,sigma_r);
        spaS = spSample(i,k);
        f_i_k(i,k) = spaS + rndS + gndS;
        if(noiseFlag_spa==1)
            nosS = normrnd(0,noiseSigma);
            f_i_k(i,k) = f_i_k(i,k) + nosS;
        end        
    end
end

% From the measurement data, estimate spatial covariance matrix
f_i_k_sqr = f_i_k.^2;
f_sample_sum = sum(f_i_k);
f_sample_sqr_sum = sum(f_i_k_sqr);

cov_F_i_j = zeros(size(spcovMvar));
for i=1:numPt
    cov_F_i_j(i,i) = (f_sample_sqr_sum(i) - f_sample_sum(i)^2/sampleN)/(sampleN-1) ;
    f_i = f_i_k(:,i);
    for j=i+1:numPt
       f_j = f_i_k(:,j);
       tmp_s_1 = sum(f_i.*f_j)/(sampleN-1);
       tmp_s_2 = f_sample_sum(i)*f_sample_sum(j)/(sampleN*(sampleN-1));
       cov_F_i_j(i,j) = tmp_s_1 - tmp_s_2;
       cov_F_i_j(j,i) = cov_F_i_j(i,j);
    end
end

if(datFromChipPtFlag==1)
  cov_F_vect = myCovM2Vect(cov_F_i_j, x1V, y1V);
else
  cov_F_vect = myCovM2Vect(cov_F_i_j, x_ptN);
end
cov_F_vect_noglb = cov_F_vect - est_sig_g_sqr;

meas_cov_f_i_j = cov_F_vect;  % measured covariance cov(Fi,Fj)
meas_cor_i_j = meas_cov_f_i_j/est_sig_F_sqr; %model correlation coeff


% solving the least square problem to find b, s, and spatial var
noguess = 1;
if(noguess==1)
  b_0 = 1;
  s_0 = 5;
  var_s_0 = 10;
  par_0 = [var_s_0, b_0, s_0]';
end
msg = sprintf('Underlying: b=%f, s=%f, var_s=%f', b, s, var_s);
disp(msg);
msg = sprintf('InitGuess: b=%f, s=%f, var_s=%f', b_0, s_0, var_s_0);
disp(msg);
var_s_lb = 0.0000001;  % only the first par in par_0 gets the bound constraint
var_s_up = est_sig_Fchip_sqr;  
[parOut,resnorm,residual] = lsqcurvefit(@mySpatialCovarianceFittingFunc, ...
    par_0, v, cov_F_vect_noglb,var_s_lb,var_s_up);
msg = sprintf('Final: b=%f, s=%f, var_s=%f', parOut(2), parOut(3), parOut(1));
disp(msg);
est_sig_s_sqr = parOut(1);
est_sig_r_sqr = est_sig_F_sqr - est_sig_g_sqr - est_sig_s_sqr; % estimate random variation component

par_b_s = [parOut(2), parOut(3)];
fitted_spa_rhov = myBesselFittingFunc(par_b_s,v);


msg = sprintf('Underlying: var(glb)=%f, var(spa)=%f, var(rnd)=%f, norm(spaRhoV)=%f',var_g,var_s,var_r, norm(rhov));
disp(msg);
msg = sprintf('Noise: var(noise)=%f, ratio = %f',noiseSigma*noiseSigma, noiseSigma*noiseSigma/var_total);
disp(msg);
msg = sprintf('Final(%d): var(glb)=%f, var(spa)=%f, var(rnd)=%f, norm(spaRhoV)=%f',...
    sampleN, est_sig_g_sqr,est_sig_s_sqr,est_sig_r_sqr, norm(fitted_spa_rhov));
disp(msg);
per_g_err = (est_sig_g_sqr-var_g)/var_g;
per_s_err = (est_sig_s_sqr-var_s)/var_s;
per_r_err = (est_sig_r_sqr-var_r)/var_r;
per_rho_err = norm(fitted_spa_rhov-rhov)/norm(rhov);
msg = sprintf('Error: var(glb)=%f, var(spa)=%f, var(rnd)=%f, norm(spaRhoV)=%f',per_g_err,...
    per_s_err,per_r_err, per_rho_err);
disp(msg);


fitted_spaCovar = mySpatialCovarianceFittingFunc(parOut,v);
fitted_cov_f_i_j = fitted_spaCovar + est_sig_g_sqr;  % fitted covariance cov(Fi,Fj)
fitted_cor_i_j = fitted_cov_f_i_j/est_sig_F_sqr; %fitted correlation coeff

% verification of fitted data use
ver_v_ptN = 10000;
ver_v_max = sqrt(2)*chipX;
ver_v_min = 0;
ver_v = linspace(ver_v_min,ver_v_max, ver_v_ptN);
ver_spaCovar = mySpatialCovarianceFittingFunc(parOut,ver_v);
ver_cov_f_i_j = ver_spaCovar + est_sig_g_sqr;  % fitted covariance cov(Fi,Fj)
ver_cor_i_j = ver_cov_f_i_j/est_sig_F_sqr; %fitted correlation coeff

% for better visibility, only plot a few points for the modeled data
[B,IX] = sort(v);
vmodel_ptN = 40; vmodel_ptN_half=vmodel_ptN/2;
ttv_ptN = length(v);
model_step = round(ttv_ptN/vmodel_ptN);
v_model = zeros(vmodel_ptN,1); model_cor_model=zeros(vmodel_ptN,1);
for i=1:vmodel_ptN_half 
    idx = 1+(i-1)*model_step; 
    v_model(i) = B(idx); 
    model_cor_model(i)=model_cor_i_j(IX(idx)); 
end
for i=(vmodel_ptN_half+1):vmodel_ptN 
    j=i-vmodel_ptN_half; 
    idx = ttv_ptN-(j-1)*model_step; 
    v_model(i) = B(idx); 
    model_cor_model(i)=model_cor_i_j(IX(idx)); 
end
figure; hold;
%plot(v,model_cor_i_j, 'r^'); %plot all model data
plot(v_model,model_cor_model, 'r^', 'LineWidth', 2);  %plot selected model data
plot(v,meas_cor_i_j,'b.');
plot(ver_v,ver_cor_i_j,'k','LineWidth', 1);
axis([ver_v_min, ver_v_max, 0, 1]);
legend('Model Data', 'Measurement Data', 'Extraction Data');

%% PWL fitting
pwlN = 3; % PWL segment number
M = [v, meas_cor_i_j];
pwlParOut = myPWL2FuncFitting(M, pwlN);

% compute spatial correlation matrix, assuming spatial vairance
% and global variance are known
est_spcovM = (cov_F_i_j - var_g)/var_s;
for i=1:numPt
    est_spcovM(i,i) = 1;
end

err_spcovM = est_spcovM - spcovM;
%mesh(err_spcovM);


    


