% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.

function my_gen_test_bench(inputSampleN, inputNoiseRatio, inputSiteN);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

initRandState;

if nargin==2
    inputSiteN = 1000;
end

%%sitePt = 1000;  % number of sites used to extract variation part
sitePt = inputSiteN;
% numPt=30;  % number of points used to extract spatial correlation function
numPt=sitePt;  % number of points used to extract spatial correlation function
chipX=10;
chipY=10;

b=2;  %underlying spatial correlation function
s=10;

var_g = 9;  sigma_g = sqrt(var_g);% global variation
var_s = 25;  sigma_s = sqrt(var_s);% spatial variation
var_r = 9;  sigma_r = sqrt(var_r); % random variation
var_total = var_g+var_s+var_r;
% msg = sprintf('Underlying: var(glb)=%f, var(spa)=%f, var(rnd)=%f',var_g,var_s,var_r);
% disp(msg);
% msg = sprintf('Underlying: var(total)=%f, var(glb)=%f, var(spatial+rnd)=%f',...
%     var_total, var_g, var_s+var_r);
% disp(msg);

noiseFlag_gen = 0;  % whether or not to add noise to the sampling to the first part
glbNoiseFlag_gen = 0;
noiseSigma = sqrt(inputNoiseRatio*var_total);  % noise is about 50% of total variation
glbNoiseSigma = 0.25*sigma_r; % glb noise

%sampleN = 1000; % # of chips
sampleN = inputSampleN; % # of chips

% generate global variation samples
gndSample = normrnd(0,sigma_g,[sampleN, 1]);

var_flag = 1;
if(var_flag == 1)
  %% generate data for variation extraction
  %% assuming all measurement sites within a chip are far away
  %% so that there is no spatial correlation between them  
  f_i_k = zeros(sampleN, sitePt);
  for i=1:sampleN    
      gndS = gndSample(i);
      if(glbNoiseFlag_gen==1)
          gndS = gndS + normrnd(0,glbNoiseSigma);
      end
      for k=1:sitePt
          rndS = normrnd(0,sigma_r);
          spaS = normrnd(0,sigma_s);
          f_i_k(i,k) = spaS + rndS + gndS;
          if(noiseFlag_gen==1)
              nosS = normrnd(0,noiseSigma);
              f_i_k(i,k) = f_i_k(i,k) + nosS;
          end        
      end
  end
  f_i_k_sqr = f_i_k.^2;
  f_sum = sum(sum(f_i_k));
  f_sqr_sum = sum(sum(f_i_k_sqr));
  est_sig_F_sqr = (f_sqr_sum - f_sum*f_sum/sampleN/sitePt)/(sampleN*sitePt-1);

  f_chip_dot_sum = sum(f_i_k, 2);
  f_sqr_chip_dot = sum(f_i_k_sqr, 2);
  est_sig_Fchip_sqr = (f_sqr_chip_dot - f_chip_dot_sum.^2/sitePt)/(sitePt-1);
  est_sig_Fchip_sqr = sum(est_sig_Fchip_sqr)/sampleN;
  est_sig_g_sqr = est_sig_F_sqr - est_sig_Fchip_sqr;
else
  est_sig_F_sqr = 42.9112;
  est_sig_g_sqr = 8.8982;
  est_sig_Fchip_sqr = 34.0131;
end
% msg = sprintf('Estimated: var(total)=%f, var(glb)=%f, var(spatial+rnd)=%f',...
%     est_sig_F_sqr, est_sig_g_sqr, est_sig_Fchip_sqr);
% disp(msg);

par_in = [b, s]';

datFromChipPtFlag = 1;
if(datFromChipPtFlag==1)
  % generate sample points from chips, which in turn determines the
  % distance
  [x1V, y1V] = mychipRndSample(numPt, chipX, chipY);
  %plot(x1V,y1V,'r.');
  % compute the correlation coefficient for the given distance v
  [v, rhov] = mySpatialEvalCorrelation(x1V, y1V, @myBesselFittingFunc, par_in);
  % put correlation into a covariance matrix format
  spcovM = myVect2CovM(rhov, x1V, y1V);
else
   % generate sample points with specified distance
   v_max = sqrt(2)*chipX;
   v_min = 0.01;
   x_ptN = 50;   
   v_ptN = x_ptN*(x_ptN-1)/2;
   v = linspace(v_min, v_max, v_ptN);
   rhov = myBesselFittingFunc(par_in,v);
   % put correlation into a covariance matrix format
   spcovM = myVect2CovM(rhov, x_ptN);
end

model_cov_f_i_j = rhov*var_s + var_g;  % model covariance cov(Fi,Fj)
model_cor_i_j = model_cov_f_i_j/(var_g+var_s+var_r); %model correlation coeff

% generate the measurement data = spatial + global + rand + noise(?)
% where spatial variation follow certain correlation 
noiseFlag_spa = 1;
glbNoiseFlag_spa = 0;
spcovMvar = spcovM.*var_s;  % get the spatial variance matrix
spmuV = zeros(1,numPt);
spSample = mvnrnd(spmuV,spcovMvar,sampleN);
for i=1:sampleN    
    gndS = gndSample(i);
    if(glbNoiseFlag_spa==1)
        gndS = gndS + normrnd(0,glbNoiseSigma);
    end
    for k=1:numPt
        rndS = normrnd(0,sigma_r);
        spaS = spSample(i,k);
        f_i_k(i,k) = spaS + rndS + gndS;
        if(noiseFlag_spa==1)
            nosS = normrnd(0,noiseSigma);
            f_i_k(i,k) = f_i_k(i,k) + nosS;
        end        
    end
end



fid = fopen('test_bench', 'w');

wafer_id=1;

% first_line='% this is a randomly generated test bench';

% fprintf(fid, first_line);
% fprintf(fid, '\n');

for i=1:sampleN
    for k=1:numPt
        tline=[num2str(f_i_k(i,k)), ' ',...
            num2str(x1V(k)), ' ',...
            num2str(y1V(k)), ' ',...
            '0 0 ',...
            num2str(wafer_id), ' 0 '];
        
        fprintf(fid, tline);            
        fprintf(fid, '\n');        
               
    end
    
    wafer_id=wafer_id+1;
end

fprintf(fid, '%');
fclose(fid);




