% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.

function [f_i_k, x1V, y1V]=my_gen_test_data(SampleN, SNR, numPt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

chipX=10;
chipY=10;
b=1;
s=10;
Fchip_var=25;
rho_s=0.4;
rho_g=0.4;
rho_r=0.2;

f_i_k = zeros(SampleN, numPt);
x1V   = zeros(numPt,1);
y1V   = zeros(numPt,1);





% generate spatial sample

[x1V, y1V] = mychipRndSample(numPt, chipX, chipY);

cov_M = zeros(numPt, numPt);
mean_vec = zeros(numPt,1);

for i=1:numPt    
    cov_M(i, i)=1;    
    for j=i+1:numPt        
        d=sqrt((x1V(i)-x1V(j))^2+(y1V(i)-y1V(j))^2);            
        cov_M(i, j)=myBesselFittingFunc([b, s].', d);
        cov_M(j, i)=cov_M(i, j);
    end
end

[V, D] = eig(cov_M);

D=abs(D);

for i_sample=1:SampleN    
    sample_tmp=normrnd(0,1,numPt,1);    
    spatial_sample(i_sample, :) = (V * sqrt(D) * sample_tmp * sqrt(rho_s * Fchip_var)).';    
end

% cov_M=V*D*V.'*rho_s * Fchip_var;

% spatial_sample=mvnrnd(mean_vec, cov_M, SampleN);

% generate random sample

random_sample = normrnd (0, 1, SampleN, numPt) * sqrt(rho_r * Fchip_var);

% generate global sample

global_sample = normrnd(0, 1, SampleN,1) * sqrt(rho_g * Fchip_var);

% generate noise

noise_sample = normrnd(0, 1, SampleN, numPt) * sqrt(Fchip_var/SNR);

% f_i_k=spatial_sample;

for i=1:SampleN
    for k=1:numPt
        f_i_k(i,k)=spatial_sample(i,k)+random_sample(i,k)+global_sample(i)+noise_sample(i,k);
    end
end

tmp=0;









