% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function [x1V, y1V] = mychipRndSample(numPt, chipX, chipY)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Given the chip size (chipX, chipY), we randomly generate a set of points
% numPt on the chip. 
%
% Example:
% >> numPt = 100; chipX = 1000; chipY = 1000;
% >> myrndcovFromBesselFunc(numPt, chipX, chipY);

if(nargin==1)
    chipX = 100;
    chipY = 100;
end
sdPtN = round(sqrt(numPt));  % average # of points on each side
sdGrdNX = round(chipX/sdPtN); % average grid size on x direction
sdGrdNY = round(chipY/sdPtN); % average grid size on y direction

% We first generate all random points with a cell grid
% Then we expand the grid to the chip size.
% This helps us to get a relatively uniform distribution
% of points in the chip
%x1V = round(rand(numPt,1)*sdGrdNX);
%y1V = round(rand(numPt,1)*sdGrdNY);
x1V = rand(numPt,1)*sdGrdNX;
y1V = rand(numPt,1)*sdGrdNY;

for i=1:numPt
    xmv = mod(i-1,sdPtN);
    ymv = floor((i-1)/sdPtN);  
    x1V(i) = x1V(i) + xmv*sdGrdNX;
    if(x1V(i)>chipX)
%        x1V(i)=chipX;
         x1V(i) = x1V(i) - sdGrdNX;
    end    
    y1V(i) = y1V(i) + ymv*sdGrdNY;
    if(y1V(i) > chipY)
%        y1V(i) = chipY;
         y1V(i) = y1V(i) - sdGrdNY;
    end
end
