% Copyright  2005-2006 The Regents of the University of California
% All Rights Reserved.
% 
% Permission to use, copy, modify, and distribute this software and its
% documentation for educational, research and non-profit purposes, without
% fee, and without a written agreement is hereby granted, provided that the
% above copyright notice, this paragraph and the following three paragraphs
% appear in all copies.
% 
% Permission to incorporate this software into commercial products may be
% obtained by contacting the University of California, Los Angeles, by
% sending email to lhe@ee.ucla.edu.
% 
% This software program and documentation are copyrighted by The Regents of
% the University of California. The software program and documentation are
% supplied "as is", without any accompanying services from The Regents. The
% Regents does not warrant that the operation of the program will be
% uninterrupted or error-free. The end-user understands that the program was
% developed for research purposes and is advised not to rely exclusively on
% the program for any reason.
% 
% IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
% DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
% LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
% DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF
% THE POSSIBILITY OF SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY
% DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
% WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
% SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
% CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
% ENHANCEMENTS, OR MODIFICATIONS.


function [x1V, y1V, covM] = myrndcovFromBesselFunc(b, s, numPt, chipX, chipY)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Robust Extraction of Spatial Correlation
%%   -by Jinjun Xiong (jinjun@ucla.edu)
%% -released on 05/30/3006
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Given the chip size (chipX, chipY), we randomly generate a set of points
% numPt on the chip. Then according to the given spatial correlation function
% (given by b and s), we generate the spatial correlation matrix and
% verify that the correlation matrix is indeed a valid correlation matrix
%
% Example:
% >> myrndcovFromBesselFunc(1,10,100);

[x1V, y1V] = mychipRndSample(numPt, chipX, chipY)

covM = zeros(numPt, numPt);
for i=1:numPt
  covM(i,i) = 1;
end

for i=1:numPt
  x1 = x1V(i);
  y1 = y1V(i);
  for j=i+1:numPt
    x2 = x1V(j);
    y2 = y1V(j);        
    d = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
    if(d<0.000000001)
      display('the distance is too close!');
%      return;
    end
    val = myBesselFittingFunc([b, s]', d);
    covM(i,j) = val;
    covM(j,i) = val;
  end
end

[R,p] = chol(covM);
if(p>0)
  display('the correlation matrix is not positive');
else
  display(' the correlation matrix is a PD');
end




function val = myexpfunc(x, maxDc)
val = exp(-x./maxDc);

